'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var api = require('pixi-sound');

/**
 * The main engine audio handler
 */

var Audio = function () {

	/**
  *
  * @param {Object} config Refer to Audio.defaultConfig
  */
	function Audio(config) {
		_classCallCheck(this, Audio);

		this.audioInstances = [];

		this._isMuted = false;

		if (PIXI.sound.isSupported === false) {
			console.error('SOUND NOT SUPPORTED');
			PIXI.sound.useLegacy = true;
		}

		if (PIXI.sound.isSupported === false) {
			throw new Error('LEGACY SOUND NOT SUPPORTED');
		}

		if (config) {
			Object.assign(this, config);
		} else {
			if (typeof Settings !== 'undefined' && Settings.audioSettings) {
				Object.assign(this, Settings.audioSettings);
			}
		}

		PIXI.sound.volumeAll = this.globalVolume;
	}

	_createClass(Audio, [{
		key: 'PlayFile',


		/**
   *
   * @param file
   * @param {number | float} volume
   * @param {boolean?} loop
   * @returns {PIXI.sound.IMediaInstance|Promise<PIXI.sound.IMediaInstance>|void|Promise<void>}
   * @constructor
   */
		value: function PlayFile(file, volume, loop) {
			'use strict';

			var self = this;
			var callbackSet = false;

			if (typeof volume === 'undefined') {
				volume = Settings.audioSettings.globalVolume;
			}
			if (typeof loop === 'undefined') {
				loop = false;
			} else if (typeof loop === 'function') {
				callbackSet = true;
			}

			if (!file) throw new Error('Audio file not found! ' + file.url);
			var instance = file.play({ loop: loop });
			if (!instance) throw new Error('Audio file failed! ' + file.url);

			instance.volume = volume;
			instance.uid = file.fileKey.generateUID();
			instance.filename = file.url;
			instance.fileKey = file.fileKey;

			this.audioInstances.push(instance);
			instance.on('end', function () {
				self.removeAudio(instance.uid);
				if (callbackSet === true) loop();
			});

			return instance;
		}

		/**
   * Stops the specified track by unique ID
   * @param {int} uid
   */

	}, {
		key: 'removeAudio',
		value: function removeAudio(uid) {
			for (var i = this.audioInstances.length - 1; i >= 0; i--) {
				if (this.audioInstances[i].uid === uid) {
					this.audioInstances[i].stop();
					this.audioInstances.splice(i, 1);
					return;
				}
			}
		}

		/**
   * Stops all instances of the specified track
   * @param {string|Array} strings
   */

	}, {
		key: 'stopAllInstancesOf',
		value: function stopAllInstancesOf(strings) {
			if (strings.hasOwnProperty && strings.hasOwnProperty('fileKey')) {
				var temp = strings;
				strings = [];
				strings.push(temp.fileKey);
			} else if (typeof strings === 'string') {
				var _temp = strings;
				strings = [];
				strings.push(_temp);
			} else if (strings.constructor === Array) {
				// handled by default
			} else {
				throw new Error('[Audio] Unhandled parameter type!');
			}

			for (var s = 0; s < strings.length; s++) {
				for (var i = this.audioInstances.length - 1; i >= 0; i--) {
					if (this.audioInstances[i].fileKey === strings[s]) {
						this.audioInstances[i].stop();
						this.audioInstances.splice(i, 1);
					}
				}
			}
		}
	}, {
		key: 'muted',


		/**
   * Gets if the global audio is muted
   * @returns {boolean}
   */
		get: function get() {
			return this._isMuted;
		}

		/**
   * Sets the global mute value
   * @param {boolean} val
   */
		,
		set: function set(val) {
			'use strict';

			if (typeof val !== 'boolean') throw new Error('[Audio] Incorrect type passed to muted');
			if (this._isMuted === val) return;

			this._isMuted = val;

			if (val === true) {
				// handle muting
				for (var i = 0; i < this.audioInstances.length; i++) {
					this.audioInstances[i]._origVolume = this.audioInstances[i].volume;
					this.audioInstances[i].volume = 0;
				}
			} else {
				// handle unmuting
				for (var _i = 0; _i < this.audioInstances.length; _i++) {
					this.audioInstances[_i].volume = this.audioInstances[_i]._origVolume ? this.audioInstances[_i]._origVolume : 1;
				}
			}
		}
	}], [{
		key: 'defaultConfig',
		get: function get() {
			return {
				globalVolume: 1
			};
		}
	}]);

	return Audio;
}();

module.exports = Audio;