'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var jsAnim = require('js-easing-functions');

var _ANIMUPDATERATE = 16;
var _COUNTERSTEP = 0.06;

var _activeAnimations = [];

var Easing = function () {
	function Easing() {
		_classCallCheck(this, Easing);
	}

	_createClass(Easing, null, [{
		key: '_pushAnim',
		value: function _pushAnim(anim) {
			Easing.activeAnimations.push(anim);
		}
	}, {
		key: '_removeAnim',
		value: function _removeAnim(uid) {
			for (var i in Easing.activeAnimations) {
				if (Easing.activeAnimations[i].uid === uid) {
					var arr = Easing.activeAnimations;
					arr.splice(i);
					Easing.activeAnimations = arr;
					return;
				}
			}
			console.warn('Anim %s not found!', uid);
		}

		/**
   * Returns true if there's an issue with the parameters that would disrupt an animation
   * @param obj
   * @param targetPos
   * @returns {boolean}
   * @private
   */

	}, {
		key: '_checkParams',
		value: function _checkParams(obj, targetPos) {
			return typeof obj.x === 'undefined' || typeof obj.y === 'undefined' || (typeof targetPos === 'undefined' ? 'undefined' : _typeof(targetPos)) !== 'object' || typeof targetPos.x === 'undefined' || typeof targetPos.y === 'undefined';
		}

		/**
   *
   * @param obj
   * @param targetPos
   * @param [speed]
   * @param [duration]
   * @param [onComplete]
   * @returns {Object}
   * @private
   */

	}, {
		key: '_generateGenericAnim',
		value: function _generateGenericAnim(obj, targetPos, speed, duration, onComplete) {
			return {
				origX: obj ? obj.x : 0,
				origY: obj ? obj.y : 0,
				targetPos: targetPos ? targetPos : { x: 0, y: 0 },
				obj: obj ? obj : null,
				speed: speed ? speed : 1.0,
				counter: 0,
				duration: duration ? duration : null,
				uid: Math.random().toString().substring(2),
				interval: null,
				onComplete: onComplete ? onComplete : function () {}
			};
		}
	}, {
		key: '_cleanupAnim',
		value: function _cleanupAnim(anim) {
			clearInterval(anim.interval);
			Easing._removeAnim(anim.uid);
			anim.onComplete();
		}

		/**
   * Behaves differently to other animations in that it does not possess a duration
   * @param {Object} obj
   * @param {Object} targetPos
   * @param {Number} [speed]
   * @param {Function} [onComplete]
   * @returns {Function}
   */

	}, {
		key: 'lerpAsync',
		value: function () {
			var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(obj, targetPos, speed, onComplete) {
				var anim;
				return regeneratorRuntime.wrap(function _callee$(_context) {
					while (1) {
						switch (_context.prev = _context.next) {
							case 0:
								if (!Easing._checkParams(obj, targetPos)) {
									_context.next = 2;
									break;
								}

								throw new Error('Invalid object passed!');

							case 2:
								anim = Easing._generateGenericAnim(obj, targetPos, speed, null, onComplete);

								anim.type = 'lerp';

								anim.interval = setInterval(function () {
									anim.obj.x = Easing.lerp(anim.origX, anim.targetPos.x, anim.counter);
									anim.obj.y = Easing.lerp(anim.origY, anim.targetPos.y, anim.counter);

									anim.counter += _COUNTERSTEP * anim.speed;

									if (anim.counter >= 1.0) anim.counter = 1.0;
									obj.x = lerp(anim.origX, anim.targetPos.x, anim.counter);
									obj.y = lerp(anim.origY, anim.targetPos.y, anim.counter);

									if (anim.counter >= 1.0) {
										anim.obj.x = anim.targetPos.x;
										anim.obj.y = anim.targetPos.y;
										clearInterval(anim.interval);
										if (anim.onComplete) anim.onComplete();
									}
								}, 16);

								return _context.abrupt('return', anim);

							case 6:
							case 'end':
								return _context.stop();
						}
					}
				}, _callee, this);
			}));

			function lerpAsync(_x, _x2, _x3, _x4) {
				return _ref.apply(this, arguments);
			}

			return lerpAsync;
		}()

		/**
   *
   * @param {Object} obj
   * @param {Object} targetPos
   * @param {Number} [speed]
   * @param {Number} [duration]
   * @param {Function} [onComplete]
   * @returns {void}
   */

	}, {
		key: 'easeInBounceAsync',
		value: function () {
			var _ref2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(obj, targetPos, speed, duration, onComplete) {
				var anim;
				return regeneratorRuntime.wrap(function _callee2$(_context2) {
					while (1) {
						switch (_context2.prev = _context2.next) {
							case 0:
								if (!Easing._checkParams(obj, targetPos)) {
									_context2.next = 2;
									break;
								}

								throw new Error('Invalid object passed!');

							case 2:
								anim = Easing._generateGenericAnim(obj, targetPos, speed, duration, onComplete);

								anim.type = 'easeInBounce';
								anim.startTime = Date.now();

								anim.interval = setInterval(function () {
									anim.obj.x = jsAnim.easeInBounce(Date.now() - anim.startTime, anim.origX, anim.targetPos.x - anim.origX, anim.duration);
									anim.obj.y = jsAnim.easeInBounce(Date.now() - anim.startTime, anim.origY, anim.targetPos.y - anim.origY, anim.duration);

									if (Date.now() - anim.startTime >= anim.duration) {
										anim.obj.x = anim.targetPos.x;
										anim.obj.y = anim.targetPos.y;
										Easing._cleanupAnim(anim);
									}
								}, _ANIMUPDATERATE);

								Easing._pushAnim(anim);

								return _context2.abrupt('return', anim);

							case 8:
							case 'end':
								return _context2.stop();
						}
					}
				}, _callee2, this);
			}));

			function easeInBounceAsync(_x5, _x6, _x7, _x8, _x9) {
				return _ref2.apply(this, arguments);
			}

			return easeInBounceAsync;
		}()

		/**
   *
   * @param {Object} obj
   * @param {Object} targetPos
   * @param {Number} [speed]
   * @param {Number} [duration]
   * @param {Function} [onComplete]
   * @returns {void}
   */

	}, {
		key: 'easeOutBounceAsync',
		value: function () {
			var _ref3 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3(obj, targetPos, speed, duration, onComplete) {
				var anim;
				return regeneratorRuntime.wrap(function _callee3$(_context3) {
					while (1) {
						switch (_context3.prev = _context3.next) {
							case 0:
								if (!Easing._checkParams(obj, targetPos)) {
									_context3.next = 2;
									break;
								}

								throw new Error('Invalid object passed!');

							case 2:
								anim = Easing._generateGenericAnim(obj, targetPos, speed, duration, onComplete);

								anim.type = 'easeOutBounce';
								anim.startTime = Date.now();

								anim.interval = setInterval(function () {
									anim.obj.x = jsAnim.easeOutBounce(Date.now() - anim.startTime, anim.origX, anim.targetPos.x - anim.origX, anim.duration);
									anim.obj.y = jsAnim.easeOutBounce(Date.now() - anim.startTime, anim.origY, anim.targetPos.y - anim.origY, anim.duration);

									if (Date.now() - anim.startTime >= anim.duration) {
										anim.obj.x = anim.targetPos.x;
										anim.obj.y = anim.targetPos.y;
										Easing._cleanupAnim(anim);
									}
								}, _ANIMUPDATERATE);

								Easing._pushAnim(anim);

								return _context3.abrupt('return', anim);

							case 8:
							case 'end':
								return _context3.stop();
						}
					}
				}, _callee3, this);
			}));

			function easeOutBounceAsync(_x10, _x11, _x12, _x13, _x14) {
				return _ref3.apply(this, arguments);
			}

			return easeOutBounceAsync;
		}()

		/**
   *
   * @param {Object} obj
   * @param {Object} targetPos
   * @param {Number} [speed]
   * @param {Number} [duration]
   * @param {Function} [onComplete]
   * @returns {void}
   */

	}, {
		key: 'easeInQuadAsync',
		value: function () {
			var _ref4 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee4(obj, targetPos, speed, duration, onComplete) {
				var anim;
				return regeneratorRuntime.wrap(function _callee4$(_context4) {
					while (1) {
						switch (_context4.prev = _context4.next) {
							case 0:
								if (!Easing._checkParams(obj, targetPos)) {
									_context4.next = 2;
									break;
								}

								throw new Error('Invalid object passed!');

							case 2:
								anim = Easing._generateGenericAnim(obj, targetPos, speed, duration, onComplete);

								anim.type = 'easeInQuad';
								anim.startTime = Date.now();

								anim.interval = setInterval(function () {
									anim.obj.x = jsAnim.easeInQuad(Date.now() - anim.startTime, anim.origX, anim.targetPos.x - anim.origX, anim.duration);
									anim.obj.y = jsAnim.easeInQuad(Date.now() - anim.startTime, anim.origY, anim.targetPos.y - anim.origY, anim.duration);

									if (Date.now() - anim.startTime >= anim.duration) {
										anim.obj.x = anim.targetPos.x;
										anim.obj.y = anim.targetPos.y;
										Easing._cleanupAnim(anim);
									}
								}, _ANIMUPDATERATE);

								Easing._pushAnim(anim);

								return _context4.abrupt('return', anim);

							case 8:
							case 'end':
								return _context4.stop();
						}
					}
				}, _callee4, this);
			}));

			function easeInQuadAsync(_x15, _x16, _x17, _x18, _x19) {
				return _ref4.apply(this, arguments);
			}

			return easeInQuadAsync;
		}()

		/**
   *
   * @param {Object} obj
   * @param {Object} targetPos
   * @param {Number} [speed]
   * @param {Number} [duration]
   * @param {Function} [onComplete]
   * @returns {void}
   */

	}, {
		key: 'easeOutQuadAsync',
		value: function () {
			var _ref5 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee5(obj, targetPos, speed, duration, onComplete) {
				var anim;
				return regeneratorRuntime.wrap(function _callee5$(_context5) {
					while (1) {
						switch (_context5.prev = _context5.next) {
							case 0:
								if (!Easing._checkParams(obj, targetPos)) {
									_context5.next = 2;
									break;
								}

								throw new Error('Invalid object passed!');

							case 2:
								anim = Easing._generateGenericAnim(obj, targetPos, speed, duration, onComplete);

								anim.type = 'easeOutQuad';
								anim.startTime = Date.now();

								anim.interval = setInterval(function () {
									anim.obj.x = jsAnim.easeOutQuad(Date.now() - anim.startTime, anim.origX, anim.targetPos.x - anim.origX, anim.duration);
									anim.obj.y = jsAnim.easeOutQuad(Date.now() - anim.startTime, anim.origY, anim.targetPos.y - anim.origY, anim.duration);

									if (Date.now() - anim.startTime >= anim.duration) {
										anim.obj.x = anim.targetPos.x;
										anim.obj.y = anim.targetPos.y;
										Easing._cleanupAnim(anim);
									}
								}, _ANIMUPDATERATE);

								Easing._pushAnim(anim);

							case 7:
							case 'end':
								return _context5.stop();
						}
					}
				}, _callee5, this);
			}));

			function easeOutQuadAsync(_x20, _x21, _x22, _x23, _x24) {
				return _ref5.apply(this, arguments);
			}

			return easeOutQuadAsync;
		}()
	}, {
		key: 'lerp',
		value: function lerp(start, end, progress) {
			return start * (1 - progress) + end * progress;
		}
	}, {
		key: 'activeAnimations',
		get: function get() {
			return _activeAnimations;
		},
		set: function set(_arr) {
			_activeAnimations = _arr;
		}
	}]);

	return Easing;
}();

module.exports = Easing;