'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var FBLeaderboards = function () {
	function FBLeaderboards(leaderboard_names, offlineMode) {
		'use strict';

		_classCallCheck(this, FBLeaderboards);

		this['debug'] = false;
		this['offlineMode'] = true;
		this['leaderboard_names'] = [];

		if (leaderboard_names) {
			if (!Array.isArray(leaderboard_names)) {
				this['leaderboard_names'] = [leaderboard_names];
			} else {
				this['leaderboard_names'] = leaderboard_names;
			}

			if (typeof offlineMode !== 'undefined' && typeof offlineMode === 'boolean') {
				this['offlineMode'] = offlineMode;
			} else if (window.Settings) {
				this['offlineMode'] = Settings.Leaderboards['offlineMode'];
			}
		} else if (window.Settings) {
			Object.assign(this, Settings.Leaderboards);
		}

		this._signedIn = false;
		this._isInit = false;
		this._leaderboards = {};
		this._initCallback = null;

		if (this.offlineMode || typeof FBInstant === 'undefined') {
			this.offlineMode = true;
		}
	}

	_createClass(FBLeaderboards, [{
		key: 'GetLeaderboard',
		value: function GetLeaderboard(name) {
			var self = this;
			return new Promise(function (resolve, reject) {
				FBInstant.getLeaderboardAsync(name).then(function (leaderboard) {
					self._leaderboards[leaderboard.getName()] = leaderboard;
					resolve(self._leaderboards[leaderboard.getName()]);
				}).catch(reject);
			});
		}

		/**
   *
   * @param {Function} callback
   */

	}, {
		key: 'init',
		value: function init(callback) {
			'use strict';

			var self = this;

			if (this.offlineMode === true) {
				this._isInit = true;
				if (callback) callback();
				return;
			}

			this._initCallback = callback ? callback : function () {};

			for (var i = 0; i < this.leaderboard_names.length; i++) {
				var boardName = this.leaderboard_names[i];

				FBInstant.getLeaderboardAsync(boardName).then(function (leaderboard) {
					self._leaderboards[leaderboard.getName()] = leaderboard;
					if (self.isInit) {
						self._isInit = true;
						self._initCallback(self);
					}
				}).catch(function (err) {
					console.error(err);
					self.offlineMode = true;
				});
			}
		}

		/**
   *
   * @param {string} boardName
   * @param {number} score
   * @param {string | object} [metadata] Must be less than 2KB
   * @returns {Promise | null}
   * @constructor
   */

	}, {
		key: 'SendScoreToLeaderboard',
		value: function SendScoreToLeaderboard(boardName, score, metadata) {
			'use strict';

			// If we're in offline mode just pretend we're done

			if (this.offlineMode) {
				return;
			}

			if ((typeof metadata === 'undefined' ? 'undefined' : _typeof(metadata)) === 'object') {
				metadata = JSON.stringify(metadata);
			}

			if (metadata.length > 2048 || typeof metadata !== 'string') {
				console.error('[FBLeaderboards] Invalid metadata!');
				return;
			}

			if (this._leaderboards[boardName]) {
				var board = this._leaderboards[boardName];

				return board.setScoreAsync(score, metadata);
			} else {
				console.error("[FBLeaderboards] Board doesn't exist!");
				return;
			}
		}

		/**
   *
   * @param {string} boardName
   * @returns {Promise | null}
   * @constructor
   */

	}, {
		key: 'GetPlayerLeaderboardInfo',
		value: function GetPlayerLeaderboardInfo(boardName) {
			'use strict';

			if (!this._leaderboards[boardName]) {
				console.error("[FBLeaderboards] Board doesn't exist!");
				return;
			}

			return this._leaderboards[boardName].getPlayerEntryAsync();
		}

		/**
   *
   * @param {string} boardName
   * @param {number} [count]
   * @param {number} [offset]
   * @returns {Promise | null}
   * @constructor
   */

	}, {
		key: 'GetLeaderboardEntries',
		value: function GetLeaderboardEntries(boardName, count, offset) {
			'use strict';

			if (!this._leaderboards[boardName]) {
				console.error("[FBLeaderboards] Board doesn't exist!");
				return;
			}

			if (!count) {
				count = 10;
			}

			if (!offset) {
				offset = 0;
			}

			return this._leaderboards[boardName].getEntriesAsync(count, offset);
		}

		/**
   * Gets all friends/connected players on the specified leaderboard
   * @param {string} boardName
   * @returns {Promise | null}
   * @constructor
   */

	}, {
		key: 'GetFriendsLeaderboardEntries',
		value: function GetFriendsLeaderboardEntries(boardName) {
			'use strict';

			if (!this._leaderboards[boardName]) {
				console.error("[FBLeaderboards] Board doesn't exist!");
				return;
			}

			return this._leaderboards[boardName].getConnectedPlayerEntriesAsync();
		}
	}, {
		key: 'isInit',
		get: function get() {
			if (this._isInit === true) return true;else {
				return Object.keys(this._leaderboards).length === this.leaderboard_names.length;
			}
		}
	}]);

	return FBLeaderboards;
}();

module.exports = FBLeaderboards;