'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var GSModule = function () {
	function GSModule(config) {
		'use strict';

		_classCallCheck(this, GSModule);

		this['debug'] = false;
		this['secret'] = null;
		this['key'] = null;
		this['offlineMode'] = null;

		if (config) {
			Object.assign(this, config);
		} else if (window.Settings && window.Settings.GameSparks) {
			Object.assign(this, Settings.GameSparks);
		} else {
			Object.assign(this, {
				key: '',
				secret: '',
				logger: console.log,
				debug: false,
				offlineMode: true
			});
		}

		this._signedIn = false;
		this._isInit = false;

		if (!this.offlineMode) {
			this._gamesparks = new GameSparks();
			/*
                Dirty hack to make GS work with build setup
             */
			global.gamesparks = this._gamesparks;
		} else this._gamesparks = {};
	}

	_createClass(GSModule, [{
		key: 'init',
		value: function init(callback) {
			'use strict';

			if (this.offlineMode === true) {
				if (callback) callback();
				return;
			}
			this._initCallback = callback;
			this._gamesparks.initPreview(this);
		}
	}, {
		key: 'GetAroundMeLeaderboard',
		value: function GetAroundMeLeaderboard(callback) {
			'use strict';

			if (!this._isInit || !this._signedIn || this.offlineMode) return;

			this._gamesparks.aroundMeLeaderboardRequest(5, [], 'global_leaderboard', false, function (data) {
				if (data.error) {
					throw new Error(data.error);
				}
				console.log('[GSModule] AroundmeLeaderboard successful');
				if (callback) callback(data.data);
			});
		}

		/**
   * Sends data to GS using key
   * @param {string} key
   * @param {string} attr
   * @param {string | number | object} data
   * @param {function?} callback
   * @constructor
   */

	}, {
		key: 'SendData',
		value: function SendData(key, attr, data, callback) {
			'use strict';

			if (this.offlineMode === true) {
				if (callback) callback();
				return;
			}
			if (this._signedIn === false) throw new Error('Not signed in!');
			if (this._isInit === false) throw new Error("Not init'd!");

			var output = {
				eventKey: key
			};
			output[attr] = data;

			this._gamesparks.sendWithData('LogEventRequest', output, function (response) {
				console.log(JSON.stringify(response));
				if (callback) callback();
			});
		}
	}, {
		key: 'onNonce',
		value: function onNonce(nonce) {
			'use strict';

			return CryptoJS.enc.Base64.stringify(CryptoJS.HmacSHA256(nonce, this['secret']));
		}
	}, {
		key: 'registerUser',
		value: function registerUser(callback) {
			'use strict';

			var _this = this;

			this._gamesparks.registrationRequest(FBINSTANT_INFO.playerInfo.displayName, FBINSTANT_INFO.playerInfo.id, FBINSTANT_INFO.playerInfo.id, function (response) {
				console.log('[GSModule] Successfully registered');
				if (_this.debug === true) console.log(JSON.stringify(response));
				if (callback) callback();
			});
		}
	}, {
		key: 'signIn',
		value: function signIn(callback) {
			'use strict';

			var _this2 = this;

			var self = this;
			if (this._signedIn) return;
			if (this._isInit === false) throw new Error('Not init!');

			this._gamesparks.authenticationRequest(FBINSTANT_INFO.playerInfo.id, FBINSTANT_INFO.playerInfo.id, function (response) {
				if (_this2.debug === true) console.log(JSON.stringify(response));

				if (response['error']) {
					switch (response['error']['DETAILS']) {
						case 'UNRECOGNISED':
							console.log('[GSModule] New user; registering...');
							self.registerUser(
							/*self.signIn(*/function () {
								if (callback) callback();
							} //)
							);
							break;
						default:
							throw new Error('Unhandled auth error!');
					}
				} else {
					console.log('[GSModule] Signed in!');
					_this2._signedIn = true;
					if (callback) callback();
				}
			});
		}
	}, {
		key: 'onInit',
		value: function onInit() {
			'use strict';

			var self = this;
			this._isInit = true;
			console.log('[GSModule] Initialized');

			this.signIn(self._initCallback ? self._initCallback : function () {});
		}
	}, {
		key: 'onMessage',
		value: function onMessage(message) {
			//if(this.debug === true)
			// console.log(JSON.stringify(message));
		}
	}, {
		key: '_gs',
		get: function get() {
			return this._gamesparks;
		}
	}]);

	return GSModule;
}();

module.exports = GSModule;